@@ -663,14 +663,24 @@ static inline void *_nl_memdup(const void *ptr, size_t len)
 
 /*****************************************************************************/
 
+static inline size_t _nl_addr_family_to_size(int addr_family)
+{
+	if (addr_family == AF_INET)
+		return sizeof(in_addr_t);
+	if (addr_family == AF_INET6)
+		return sizeof(struct in6_addr);
+	return 0;
+}
+
+/*****************************************************************************/
+
 typedef union {
 	in_addr_t addr4;
 	struct in_addr a4;
 	struct in6_addr a6;
 } _NLIPAddr;
 
-static inline char *_nl_inet_ntop(int addr_family, const void *addr,
-				  char buf[static INET_ADDRSTRLEN])
+static inline char *_nl_inet_ntop(int addr_family, const void *addr, char *buf)
 {
 	char *r;
 
@@ -724,6 +734,14 @@ static inline char *_nl_inet_ntop_dup(int addr_family, const void *addr)
 	}                                                \
 	struct _nl_dummy_for_tailing_semicolon
 
+#define _NL_AUTO_DEFINE_FCN_INDIRECT0(CastType, name, func) \
+	static inline void name(CastType *v)                \
+	{                                                   \
+		if (*v)                                     \
+			func(v);                            \
+	}                                                   \
+	struct _nl_dummy_for_tailing_semicolon
+
 #define _nl_auto_free _nl_auto(_nl_auto_free_fcn)
 _NL_AUTO_DEFINE_FCN_VOID0(void *, _nl_auto_free_fcn, free);
 
