@@ -1,11 +1,5 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * netlink/route/sch/red.h	RED Qdisc
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2003-2006 Thomas Graf <tgraf@suug.ch>
  */
 
@@ -13,8 +7,17 @@
 #define NETLINK_RED_H_
 
 #include <netlink/netlink.h>
+#include <netlink/route/qdisc.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 extern  void rtnl_red_set_limit(struct rtnl_qdisc *qdisc, int limit);
 extern int rtnl_red_get_limit(struct rtnl_qdisc *qdisc);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
