@@ -21,11 +21,17 @@
  * ~~~~
  */
 
-#include <netlink-private/netlink.h>
-#include <netlink/netlink.h>
-#include <netlink/utils.h>
+#include "nl-default.h"
+
 #include <linux/socket.h>
 
+#include <netlink/netlink.h>
+#include <netlink/data.h>
+#include <netlink/attr.h>
+#include <netlink/utils.h>
+
+#include "nl-priv-dynamic-core/nl-core.h"
+
 /**
  * @name General
  * @{
