@@ -1,16 +1,10 @@
 /* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lib/error.c		Error Handling
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2008 Thomas Graf <tgraf@suug.ch>
  */
 
-#include <netlink-private/netlink.h>
+#include "nl-default.h"
+
 #include <netlink/netlink.h>
 
 static const char *errmsg[NLE_MAX+1] = {
