@@ -73,7 +73,6 @@
 #define LINK_ATTR_GSO_MAX_SEGS		((uint64_t) 1 << 37)
 #define LINK_ATTR_GSO_MAX_SIZE		((uint64_t) 1 << 38)
 #define LINK_ATTR_LINKINFO_SLAVE_KIND	((uint64_t) 1 << 39)
-#define LINK_ATTR_PERMANENT_ADDR	((uint64_t) 1 << 40)
 
 static struct nl_cache_ops rtnl_link_ops;
 static struct nl_object_ops link_obj_ops;
@@ -272,7 +271,6 @@ static void link_free_data(struct nl_object *c)
 
 		nl_addr_put(link->l_addr);
 		nl_addr_put(link->l_bcast);
-		nl_addr_put(link->l_paddr);
 
 		free(link->l_ifalias);
 		free(link->l_info_kind);
@@ -296,7 +294,6 @@ static int link_clone(struct nl_object *_dst, struct nl_object *_src)
 
 	dst->l_addr = NULL;
 	dst->l_bcast = NULL;
-	dst->l_paddr = NULL;
 	dst->l_info_kind = NULL;
 	dst->l_info_slave_kind = NULL;
 	dst->l_info_ops = NULL;
@@ -316,10 +313,6 @@ static int link_clone(struct nl_object *_dst, struct nl_object *_src)
 		if (!(dst->l_bcast = nl_addr_clone(src->l_bcast)))
 			return -NLE_NOMEM;
 
-	if (src->l_paddr)
-		if (!(dst->l_paddr = nl_addr_clone(src->l_paddr)))
-			return -NLE_NOMEM;
-
 	if (src->l_ifalias)
 		if (!(dst->l_ifalias = strdup(src->l_ifalias)))
 			return -NLE_NOMEM;
@@ -447,7 +440,7 @@ int rtnl_link_info_parse(struct rtnl_link *link, struct nlattr **tb)
 		/* beware: @st might not be the full struct, only fields up to
 		 * tx_compressed are present. See _nl_offsetofend() above. */
 
-		if (_nla_len(tb[IFLA_STATS]) >= _nl_offsetofend (struct rtnl_link_stats, rx_nohandler))
+		if (nla_len(tb[IFLA_STATS]) >= _nl_offsetofend (struct rtnl_link_stats, rx_nohandler))
 			link->l_stats[RTNL_LINK_RX_NOHANDLER] = st->rx_nohandler;
 		else
 			link->l_stats[RTNL_LINK_RX_NOHANDLER] = 0;
@@ -521,15 +514,6 @@ int rtnl_link_info_parse(struct rtnl_link *link, struct nlattr **tb)
 		link->ce_mask |= LINK_ATTR_ADDR;
 	}
 
-	if (tb[IFLA_PERM_ADDRESS]) {
-		link->l_paddr = nl_addr_alloc_attr(tb[IFLA_PERM_ADDRESS], AF_UNSPEC);
-		if (link->l_paddr == NULL)
-			return -NLE_NOMEM;
-		nl_addr_set_family(link->l_paddr,
-				   nl_addr_guess_family(link->l_paddr));
-		link->ce_mask |= LINK_ATTR_PERMANENT_ADDR;
-	}
-
 	if (tb[IFLA_BROADCAST]) {
 		link->l_bcast = nl_addr_alloc_attr(tb[IFLA_BROADCAST],
 						   AF_UNSPEC);
@@ -697,7 +681,11 @@ static int link_msg_parser(struct nl_cache_ops *ops, struct sockaddr_nl *who,
 					tb[IFLA_PROTINFO] = (struct nlattr *)link->l_af_ops->ao_protinfo_policy;
 			}
 
-			ops = link->l_info_ops;
+			if (link->l_info_ops)
+				release_link_info(link);
+
+			ops = rtnl_link_info_ops_lookup(kind);
+			link->l_info_ops = ops;
 
 			if (ops) {
 				if (ops->io_parse &&
@@ -879,9 +867,6 @@ static void link_dump_line(struct nl_object *obj, struct nl_dump_params *p)
 	if (link->l_addr && !nl_addr_iszero(link->l_addr))
 		nl_dump(p, "%s ", nl_addr2str(link->l_addr, buf, sizeof(buf)));
 
-	if (link->l_paddr && !nl_addr_iszero(link->l_paddr))
-		nl_dump(p, "permanent address %s ", nl_addr2str(link->l_paddr, buf, sizeof(buf)));
-
 	if (link->ce_mask & LINK_ATTR_MASTER) {
 		if (cache) {
 			_nl_auto_rtnl_link struct rtnl_link *master = rtnl_link_get(cache, link->l_master);
@@ -1149,7 +1134,6 @@ static uint64_t link_compare(struct nl_object *_a, struct nl_object *_b,
 	diff |= _DIFF(LINK_ATTR_IFNAME, strcmp(a->l_name, b->l_name));
 	diff |= _DIFF(LINK_ATTR_ADDR, nl_addr_cmp(a->l_addr, b->l_addr));
 	diff |= _DIFF(LINK_ATTR_BRD, nl_addr_cmp(a->l_bcast, b->l_bcast));
-	diff |= _DIFF(LINK_ATTR_PERMANENT_ADDR, nl_addr_cmp(a->l_paddr, b->l_paddr));
 	diff |= _DIFF(LINK_ATTR_IFALIAS, strcmp(a->l_ifalias, b->l_ifalias));
 	diff |= _DIFF(LINK_ATTR_NUM_VF, a->l_num_vf != b->l_num_vf);
 	diff |= _DIFF(LINK_ATTR_PROMISCUITY,
@@ -1336,7 +1320,7 @@ struct rtnl_link *rtnl_link_get(struct nl_cache *cache, int ifindex)
 		return NULL;
 
 	nl_list_for_each_entry(link, &cache->c_items, ce_list) {
-		if (link->l_index == ((unsigned)ifindex)) {
+		if (link->l_index == ifindex) {
 			nl_object_get((struct nl_object *) link);
 			return link;
 		}
@@ -2044,19 +2028,6 @@ struct nl_addr *rtnl_link_get_addr(struct rtnl_link *link)
 	return link->ce_mask & LINK_ATTR_ADDR ? link->l_addr : NULL;
 }
 
-/**
- * Return permanent link layer adress of link object
- * @arg link		Link object
- *
- * @copydoc pointer_lifetime_warning
- * @route_doc{link_attr_permaddr, Permanent Link Layer Address}
- * @return Permanent link layer address or NULL if not set.
- */
-struct nl_addr *rtnl_link_get_perm_addr(struct rtnl_link *link)
-{
-	return link->ce_mask & LINK_ATTR_PERMANENT_ADDR ? link->l_paddr : NULL;
-}
-
 /**
  * Set link layer broadcast address of link object
  * @arg link		Link object
