@@ -29,7 +29,6 @@
 
 #include "nl-route.h"
 #include "link-api.h"
-#include "nl-aux-route/nl-route.h"
 
 #define IPIP_ATTR_LINK          (1 << 0)
 #define IPIP_ATTR_LOCAL         (1 << 1)
@@ -190,12 +189,10 @@ static void ipip_dump_line(struct rtnl_link *link, struct nl_dump_params *p)
 static void ipip_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 {
 	struct ipip_info *ipip = link->l_info;
-	char addr[INET_ADDRSTRLEN];
+	char *name, addr[INET_ADDRSTRLEN];
+	struct rtnl_link *parent;
 
 	if (ipip->ipip_mask & IPIP_ATTR_LINK) {
-		_nl_auto_rtnl_link struct rtnl_link *parent = NULL;
-		char *name;
-
 		nl_dump(p, "      link ");
 
 		name = NULL;
@@ -211,14 +208,18 @@ static void ipip_dump_details(struct rtnl_link *link, struct nl_dump_params *p)
 
 	if (ipip->ipip_mask & IPIP_ATTR_LOCAL) {
 		nl_dump(p, "      local ");
-		_nl_inet_ntop4(ipip->local, addr);
-		nl_dump_line(p, "%s\n", addr);
+		if(inet_ntop(AF_INET, &ipip->local, addr, sizeof(addr)))
+			nl_dump_line(p, "%s\n", addr);
+		else
+			nl_dump_line(p, "%#x\n", ntohs(ipip->local));
 	}
 
 	if (ipip->ipip_mask & IPIP_ATTR_REMOTE) {
 		nl_dump(p, "      remote ");
-		_nl_inet_ntop4(ipip->remote, addr);
-		nl_dump_line(p, "%s\n", addr);
+		if(inet_ntop(AF_INET, &ipip->remote, addr, sizeof(addr)))
+			nl_dump_line(p, "%s\n", addr);
+		else
+			nl_dump_line(p, "%#x\n", ntohs(ipip->remote));
 	}
 
 	if (ipip->ipip_mask & IPIP_ATTR_TTL) {
