@@ -1,17 +1,13 @@
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * tests/check-all.c		overall unit test program
- *
- *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU Lesser General Public
- *	License as published by the Free Software Foundation version 2.1
- *	of the License.
- *
  * Copyright (c) 2013 Thomas Graf <tgraf@suug.ch>
  */
 
+#include "nl-default.h"
+
 #include <check.h>
 
-#include "util.h"
+#include "cksuite-all.h"
 
 static Suite *main_suite(void)
 {
@@ -24,16 +20,13 @@ int main(int argc, char *argv[])
 {
 	SRunner *runner;
 	int nfailed;
-	
-	runner = srunner_create(main_suite());
 
-	/* Add testsuites below */
+	runner = srunner_create(main_suite());
 
 	srunner_add_suite(runner, make_nl_addr_suite());
 	srunner_add_suite(runner, make_nl_attr_suite());
 	srunner_add_suite(runner, make_nl_ematch_tree_clone_suite());
-
-	/* Do not add testsuites below this line */
+	srunner_add_suite(runner, make_nl_netns_suite());
 
 	srunner_run_all(runner, CK_ENV);
 
