@@ -3,18 +3,15 @@
 #ifndef __NL_TEST_UTIL_H__
 #define __NL_TEST_UTIL_H__
 
-#include <errno.h>
 #include <sys/stat.h>
 #include <check.h>
-#include <string.h>
-#include <stdbool.h>
-#include <arpa/inet.h>
 
-#include "netlink/object.h"
-#include "netlink/cache.h"
+#include <netlink/object.h>
+#include <netlink/cache.h>
 
-#include "netlink-private/nl-auto.h"
-#include "netlink-private/utils.h"
+#include "base/nl-base-utils.h"
+#include "nl-aux-core/nl-core.h"
+#include "nl-aux-route/nl-route.h"
 
 /*****************************************************************************/
 
